@@ -3,8 +3,8 @@
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
 
-#ifndef SOUP_CONNECTION_H
-#define SOUP_CONNECTION_H 1
+#ifndef __SOUP_CONNECTION_H__
+#define __SOUP_CONNECTION_H__ 1
 
 #include "soup-types.h"
 #include "soup-message-private.h"
@@ -36,9 +36,9 @@ GType soup_connection_get_type (void);
 
 
 #define SOUP_CONNECTION_REMOTE_URI        "remote-uri"
-#define SOUP_CONNECTION_SSL_FALLBACK      "ssl-fallback"
 #define SOUP_CONNECTION_SOCKET_PROPERTIES "socket-properties"
 #define SOUP_CONNECTION_STATE             "state"
+#define SOUP_CONNECTION_SSL               "ssl"
 
 void            soup_connection_connect_async    (SoupConnection       *conn,
 						  GCancellable         *cancellable,
@@ -80,8 +80,6 @@ void            soup_connection_send_request   (SoupConnection          *conn,
 						SoupMessageCompletionFn  completion_cb,
 						gpointer                 user_data);
 
-gboolean        soup_connection_get_ssl_fallback (SoupConnection   *conn);
-
 G_END_DECLS
 
-#endif /* SOUP_CONNECTION_H */
+#endif /* __SOUP_CONNECTION_H__ */
