@@ -4,8 +4,8 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#ifndef SOUP_MESSAGE_QUEUE_H
-#define SOUP_MESSAGE_QUEUE_H 1
+#ifndef __SOUP_MESSAGE_QUEUE_H__
+#define __SOUP_MESSAGE_QUEUE_H__ 1
 
 #include "soup-connection.h"
 #include "soup-message.h"
@@ -48,9 +48,11 @@ struct _SoupMessageQueueItem {
 	guint new_api           : 1;
 	guint io_started        : 1;
 	guint async             : 1;
+	guint async_pending     : 1;
 	guint conn_is_dedicated : 1;
+	guint connect_only      : 1;
 	guint priority          : 3;
-	guint resend_count      : 25;
+	guint resend_count      : 5;
 
 	SoupMessageQueueItemState state;
 
@@ -84,4 +86,4 @@ void soup_message_queue_item_unref          (SoupMessageQueueItem *item);
 
 G_END_DECLS
 
-#endif /* SOUP_MESSAGE_QUEUE_H */
+#endif /* __SOUP_MESSAGE_QUEUE_H__ */
