@@ -5,6 +5,10 @@
  * Copyright 1999-2003 Ximian, Inc.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -29,7 +33,7 @@
  * @scheme: the URI scheme (eg, "http")
  * @user: a username, or %NULL
  * @password: a password, or %NULL
- * @host: the hostname or IP address
+ * @host: the hostname or IP address, or %NULL
  * @port: the port number on @host
  * @path: the path on @host
  * @query: a query for @path, or %NULL
@@ -220,13 +224,13 @@ soup_scheme_default_port (const char *scheme)
 }
 
 /**
- * soup_uri_new_with_base:
+ * soup_uri_new_with_base: (constructor)
  * @base: a base URI
  * @uri_string: the URI
  *
  * Parses @uri_string relative to @base.
  *
- * Return value: a parsed #SoupURI.
+ * Returns: a parsed #SoupURI.
  **/
 SoupURI *
 soup_uri_new_with_base (SoupURI *base, const char *uri_string)
@@ -449,7 +453,7 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		}
 		/* Remove "<segment>/.." at end where <segment> != ".." */
 		q = strrchr (uri->path, '/');
-		if (q && !strcmp (q, "/..")) {
+		if (q && q != uri->path && !strcmp (q, "/..")) {
 			p = q - 1;
 			while (p > uri->path && *p != '/')
 				p--;
@@ -526,7 +530,7 @@ soup_uri_new (const char *uri_string)
 
 char *
 soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
-			     gboolean force_port)
+			     gboolean include_password, gboolean force_port)
 {
 	GString *str;
 	char *return_result;
@@ -542,6 +546,10 @@ soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
 		g_string_append (str, "//");
 		if (uri->user) {
 			append_uri_encoded (str, uri->user, ":;@?/");
+			if (uri->password && include_password) {
+				g_string_append_c (str, ':');
+				append_uri_encoded (str, uri->password, ";@?/");
+			}
 			g_string_append_c (str, '@');
 		}
 		if (strchr (uri->host, ':')) {
@@ -607,7 +615,7 @@ soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
 char *
 soup_uri_to_string (SoupURI *uri, gboolean just_path_and_query)
 {
-	return soup_uri_to_string_internal (uri, just_path_and_query, FALSE);
+	return soup_uri_to_string_internal (uri, just_path_and_query, FALSE, FALSE);
 }
 
 /**
@@ -746,6 +754,8 @@ soup_uri_encode (const char *part, const char *escape_extra)
 #define XDIGIT(c) ((c) <= '9' ? (c) - '0' : ((c) & 0x4F) - 'A' + 10)
 #define HEXCHAR(s) ((XDIGIT (s[1]) << 4) + XDIGIT (s[2]))
 
+/* length must be set (e.g. from strchr()) such that [part, part + length]
+ * contains no nul bytes */
 char *
 soup_uri_decoded_copy (const char *part, int length, int *decoded_length)
 {
@@ -758,7 +768,9 @@ soup_uri_decoded_copy (const char *part, int length, int *decoded_length)
 	s = d = (unsigned char *)decoded;
 	do {
 		if (*s == '%') {
-			if (!g_ascii_isxdigit (s[1]) ||
+			if (s[1] == '\0' ||
+			    s[2] == '\0' ||
+			    !g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
 				*d++ = *s;
 				continue;
@@ -795,6 +807,8 @@ soup_uri_decode (const char *part)
 	return soup_uri_decoded_copy (part, strlen (part), NULL);
 }
 
+/* length must be set (e.g. from strchr()) such that [part, part + length]
+ * contains no nul bytes */
 static char *
 uri_normalized_copy (const char *part, int length,
 		     const char *unescape_extra)
@@ -809,7 +823,9 @@ uri_normalized_copy (const char *part, int length,
 	s = d = (unsigned char *)normalized;
 	while (*s) {
 		if (*s == '%') {
-			if (!g_ascii_isxdigit (s[1]) ||
+			if (s[1] == '\0' ||
+			    s[2] == '\0' ||
+			    !g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
 				*d++ = *s++;
 				continue;
@@ -864,7 +880,8 @@ uri_normalized_copy (const char *part, int length,
  * @unescape_extra: (allow-none): reserved characters to unescape (or %NULL)
  *
  * %<!-- -->-decodes any "unreserved" characters (or characters in
- * @unescape_extra) in @part.
+ * @unescape_extra) in @part, and %<!-- -->-encodes any non-ASCII
+ * characters, spaces, and non-printing characters in @part.
  *
  * "Unreserved" characters are those that are not allowed to be used
  * for punctuation according to the URI spec. For example, letters are
@@ -1325,9 +1342,11 @@ soup_uri_is_http (SoupURI *uri, char **aliases)
 {
 	int i;
 
-	if (uri->scheme == SOUP_URI_SCHEME_HTTP)
+	if (uri->scheme == SOUP_URI_SCHEME_HTTP ||
+	    uri->scheme == SOUP_URI_SCHEME_WS)
 		return TRUE;
-	else if (uri->scheme == SOUP_URI_SCHEME_HTTPS)
+	else if (uri->scheme == SOUP_URI_SCHEME_HTTPS ||
+		 uri->scheme == SOUP_URI_SCHEME_WSS)
 		return FALSE;
 	else if (!aliases)
 		return FALSE;
@@ -1337,7 +1356,7 @@ soup_uri_is_http (SoupURI *uri, char **aliases)
 			return TRUE;
 	}
 
-	if (!aliases[1] && !strcmp (aliases[0], "*"))
+	if (aliases[0] && !aliases[1] && !strcmp (aliases[0], "*"))
 		return TRUE;
 	else
 		return FALSE;
@@ -1348,9 +1367,11 @@ soup_uri_is_https (SoupURI *uri, char **aliases)
 {
 	int i;
 
-	if (uri->scheme == SOUP_URI_SCHEME_HTTPS)
+	if (uri->scheme == SOUP_URI_SCHEME_HTTPS ||
+	    uri->scheme == SOUP_URI_SCHEME_WSS)
 		return TRUE;
-	else if (uri->scheme == SOUP_URI_SCHEME_HTTP)
+	else if (uri->scheme == SOUP_URI_SCHEME_HTTP ||
+		 uri->scheme == SOUP_URI_SCHEME_WS)
 		return FALSE;
 	else if (!aliases)
 		return FALSE;
