@@ -131,6 +131,19 @@ debug_printf (int level, const char *format, ...)
 	va_end (args);
 }
 
+gboolean
+have_curl(void) {
+	char *found;
+
+	found = g_find_program_in_path ("curl");
+	if (found) {
+		g_free (found);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 #ifdef HAVE_APACHE
 
 static gboolean
@@ -189,7 +202,9 @@ apache_cmd (const char *cmd)
 void
 apache_init (void)
 {
-	if (g_getenv ("SOUP_TESTS_IN_MAKE_CHECK"))
+	/* Set this environment variable if you are already running a
+	 * suitably-configured Apache server */
+	if (g_getenv ("SOUP_TESTS_ALREADY_RUNNING_APACHE"))
 		return;
 
 	if (!apache_cmd ("start")) {
@@ -239,9 +254,13 @@ soup_test_session_new (GType type, ...)
 	va_end (args);
 
 	if (tls_available) {
+		char *abs_cafile;
+
 		cafile = g_test_build_filename (G_TEST_DIST, "test-cert.pem", NULL);
-		tlsdb = g_tls_file_database_new (cafile, &error);
+		abs_cafile = g_canonicalize_filename (cafile, NULL);
 		g_free (cafile);
+		tlsdb = g_tls_file_database_new (abs_cafile, &error);
+		g_free (abs_cafile);
 		if (error) {
 			if (g_strcmp0 (g_getenv ("GIO_USE_TLS"), "dummy") == 0)
 				g_clear_error (&error);
